/*
 * CPSC 601.28 - Lecture 3 - Lines and Curves
 * Code by David Ledo (dledomai@ucalgary.ca)
 * To run each example/exercise, uncomment desired line and run code
 * Make sure you comment out the other function calls
 */

void setup()
{
  
  size(900, 900);
  
  // arguments are r, g, b, 0 to 255
  background(255, 255, 255);
  
  
}

/*
 * The following lines of code show individual examples and exercises as functions
 * each function is commented to explain what it does and parameters/arguments (if any)
 */

/**
 * Draws a red line with a specific stroke weight
 * by using standard processing functions such as line() and strokeWeight()
 *
 * parameters:
 * x1 = x-coordinate of starting point of line
 * y1 = y-coordinate of starting point of line
 * x2 = x-coordinate of ending point of line
 * y2 = y-coordinate of ending point of line
 * strokeWeight = the desired weight for the stroke
 */
void example1(int x1, int y1, int x2, int y2, int strokeWeight)
{
  stroke(190, 30, 45);
  strokeWeight(strokeWeight);
  line(x1, y1, x2, y2);
}

/**
 * Draws a blue line that crosses a red line
 */
void exercise1()
{
  // blue line
  stroke(0,0,255);
  strokeWeight(2);
  line(20, 20, 800, 800);
  
  //red line
  stroke(255,0,0);
  strokeWeight(20);
  line(800, 20, 20, 800);
}

/**
 * Draws a rectagle with a thick gray border
 *
 */
void exercise2()
{
  // picking the tools
  fill(255, 255, 255); 
  stroke(100, 100, 100);
  strokeWeight(10);
  
  // using the tools
  rect(20, 20, 100, 100);
}

/**
 * Draws 20 lines through a loop that goes from 0 to 19 -
 * each line has a different weight as determined by our counter (i)
 */
void example2()
{
  stroke(190, 30, 45);
  
  // loop from 0 to 19
  for(int i = 0; i < 20; i++)
  {
    strokeWeight(i/2 + 1);
    line(400, 400, i * 50, 400 + i * 50); 
  }
 
}

/**
 * Draws a path or shape.
 * Remember that paths have a fill by default.
 * Use commands such as noFill() or noStroke() to change this.
 */
void example3()
{
   stroke(190, 30, 45);
   strokeWeight(10);
   fill(100, 100, 100);
   // noFill();
   
   // draw a path
   beginShape();
   vertex(20, 20);
   vertex(20, 500);
   vertex(600, 800);
   vertex(20, 20);
   endShape();
}

/**
 * Draws a curve (2 points and 2 control points)
 * Notice the use of variable names - they increase:
 * (1) readability
 * (2) code reuse, as shown when drawing the ellipse
 */
void example4()
{
  noFill();
  stroke(190, 30, 45); 
  strokeWeight(10);
  int cx0 = 50;
  int cy0 = 700;
  int x0 = 300;
  int y0 = 300;
  int x1 = 700;
  int y1 = 300;
  int cx1 = 850;
  int cy1 = 500;
  curve(cx0, cy0, x0, y0, x1, y1, cx1, cy1);
    
  // draw control points to see
  fill(100,100,100);
  noStroke();
  ellipse(cx0, cy0, 20, 20);
  ellipse(cx1, cy1, 20, 20);
  
}

/**
 * Draws a curve path
 * Note that unlike straight paths we need control points
 */
void example5()
{
  noFill();
  stroke(190, 30, 45); 
  strokeWeight(10);
  
     beginShape();
    // start control point
    curveVertex(100, 400);
    
    // start point of curve
    curveVertex(100, 400);
    curveVertex(200, 300);
    curveVertex(400, 500);
    // end of curve
    curveVertex(500, 400);
    
    // end control point
    curveVertex(500, 400);
    endShape();
 
}

/**
 * Draws a Bezier Curve
 * note the use of variables and how the order of parameters is
 * different from curve(). This means that if we changed our minds
 * and wanted to use curve() instead of bezier(), it would be easier
 * to reshuffle the variables.
 */
void example6()
{
  noFill();
  stroke(190, 30, 45); 
  strokeWeight(10);
  
  int cx0 = 0;
  int cy0 = 900;
  int x0 = 200;
  int y0 = 400;
  int x1 = 600;
  int y1 = 400;
  int cx1 = 800;
  int cy1 = 900; 
  
  // this line of code does exactly the same thing
  // notice how it's harder to read?
  // bezier(200, 400, 0, 900, 800, 900, 600, 400);
  
  bezier(x0, y0, cx0, cy0, cx1, cy1, x1, y1);
 
}

/*
 * Draw function
 * this is where we run our code!
 * Uncomment an individual line to run each example.
 */
void draw()
{

//  background(255, 255, 255);
//  fill(100, 100, 100);
//  rect(mouseX, mouseY, 10, 10);

//  example1(20, 20, 800, 800, 1);
//  example1(40, 20, 800, 600, 10);
  
//  exercise1();
//  exercise2();

//  example2();
//  example3();
//  example4();
//  example5();
  example6();

}
